/*
youtube.c - YouTube hack for RSStool

Copyright (c) 2006 NoisyB


This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef  HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <time.h>
#include "misc/net.h"
#include "misc/string.h"
#include "misc/rss.h"
#include "misc/misc.h"
#include "rsstool_defines.h"
#include "rsstool.h"
#include "rsstool_misc.h"
#include "youtube.h"


#define YOUTUBE_URL_S "http://www.youtube.com"
#define YOUTUBE_SEARCH_S YOUTUBE_URL_S "/results?search_type=search_videos&search_query=%s&search_sort=video_date_uploaded&search_category=0"
#define YOUTUBE_HOOK1 "/watch?"
#define YOUTUBE_HOOK2 "/player2.swf?"


static int
get_movie (const char *str)
{
  st_net_t *n = NULL;
  char buf[MAXBUFSIZE];
  char o_url[MAXBUFSIZE];
  char *p = NULL;
  const char *u = net_tag_get_value (str, "href");

#ifdef  DEBUG
  printf ("%s\n", u);
#endif

  sprintf (buf, YOUTUBE_URL_S "%s", u);
  strcpy (o_url, buf);

  if (!(n = net_init (0)))
    {
      fprintf (stderr, "ERROR: rsstool_rss_zophar()/net_init() failed\n");
      return -1;
    }

  if (net_open (n, YOUTUBE_URL_S, 80) != 0)
    {
      fprintf (stderr, "ERROR: could not log on to %s\n", YOUTUBE_URL_S);
      return -1;
    }

  p = net_build_http_request (buf, rsstool.user_agent, 0, NET_METHOD_GET, 0);
  net_write (n, (char *) p, strlen (p));

  while (net_gets (n, buf, MAXBUFSIZE))
    if (strstr (buf, YOUTUBE_HOOK2))
      {
        p = strchr (buf, '\n');
        if (p)
          *p = 0;

        strtrim_s (buf, YOUTUBE_HOOK2, NULL);

        p = strchr (buf, '"');
        if (p)
          *p = 0;

        strins (buf, YOUTUBE_URL_S "/get_video?");

#ifdef  DEBUG
        printf ("%s\n", buf);
#endif
        strins (o_url, "<a href=\"");
        strcat (o_url, "\">Original URL</a>");

        rsstool_add_item_s (&rsstool,
                            "YouTube",
                            YOUTUBE_URL_S,
                            time (0),
                            buf,
                            buf,
                            o_url);
        break;
      }

  net_close (n);

  net_quit (n);

  return 0;
}


static const char *
a_pass (const char *str)
{
  static int count = 0;

  if (strstr (str, YOUTUBE_HOOK1))
    {
      if (!count)
        get_movie (str);
      count = !count;
    }

  return "";
}


st_tag_filter_t youtube_item_filter[] = {
  {
    "a",
    a_pass
  },
  {NULL, NULL}
};


int
youtube_get_rss (st_rsstool_t *rt)
{
  st_net_t *n = NULL;
  char buf[MAXBUFSIZE];
  char *p = NULL;
  int cf = 0;

  if (!(n = net_init (0)))
    {
      fprintf (stderr, "ERROR: youtube_get_rss()/net_init() failed\n");
      return -1;
    }

  if (net_open (n, YOUTUBE_URL_S, 80) != 0)
    {
      fprintf (stderr, "ERROR: could not log on to %s\n", YOUTUBE_URL_S);
      return -1;
    }

  sprintf (buf, YOUTUBE_SEARCH_S, rt->optarg);
  p = net_build_http_request (buf, rsstool.user_agent, 0, NET_METHOD_GET, 0);
  net_write (n, (char *) p, strlen (p));

  while (net_gets (n, buf, MAXBUFSIZE))
    {
      p = strchr (buf, '\n');
      if (p)
        *p = 0;

      cf = net_tag_filter (buf, youtube_item_filter, cf);
    }

  net_close (n);

  net_quit (n);

  return 0;
}
