/*
  simple wrapper for ODBC or libmysql
  
  Copyright (c) 2006 Dirk
                            
                            
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef  HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef  USE_MYSQL
#include <mysql/mysql.h>
#include "sql.h"


st_sql_t *
sql_mysql_open (st_sql_t *sql,
                const char *host, int port,
                const char *db, const char *user,
                const char *password)
{
  mysql_init (&sql->db);
  mysql_options (&sql->db, MYSQL_READ_DEFAULT_GROUP, "random");

  if (!mysql_real_connect (&sql->db, host, user, password, db, port, NULL, 0))
    {
      fprintf (stderr, "ERROR: mysql_real_connect(): %s\n", mysql_error (&sql->db));
      return NULL;
    }

#if 1
  if (mysql_select_db (&sql->db, db))  // select the database we want to use
    {
      fprintf (stderr, "ERROR: mysql_select_db(): %s\n", mysql_error (&sql->db));
      return NULL;
    }
#endif

  return sql;
}


int
sql_mysql_close (st_sql_t *sql)
{
  if (sql->res)
    {
      mysql_free_result (sql->res);
      sql->res = NULL;
    }

  mysql_close (&sql->db);

  return 0;
}


int
sql_mysql_query (st_sql_t *sql, const char *query)
{
  if (mysql_real_query (&sql->db, query, strlen (query)))
    {
      fprintf (stderr, "ERROR: mysql_real_query(): %s\n", mysql_error (&sql->db));
      return -1;
    }

  sql->res = mysql_store_result (&sql->db);  // download result from server

  return 0;
}


char *
sql_mysql_gets (st_sql_t *sql, char *buf, int buf_len)
{
  if (sql->res)
    {
      if ((sql->row = mysql_fetch_row (sql->res)))
        {
          strncpy (buf, sql->row[0], buf_len)[buf_len - 1] = 0;
          return buf;
        }

      mysql_free_result (sql->res);
      sql->res = NULL;
    }

  return NULL;
}
#endif  // USE_MYSQL
