/*
  simple wrapper for ODBC or libmysql
  
  Copyright (c) 2006 Dirk
                            
                            
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef  HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef  USE_ODBC
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sql.h>
#include <sqlext.h>
#include <sqltypes.h>
#include "sql.h"


#define MAXBUFSIZE 32768


#if 0
static char *
uc_to_ascii (SQLWCHAR *ws)
{
  char *s = (char *) ws;
  int i = 0;

  for (; ws[i]; i++)
    s[i] = ws[i] & 0x00ff;

  s[i] = 0;

  return s;
}


static void
ascii_to_uc (char *s, SQLWCHAR *ws)
{
  int i = 0;

  for (; s[i]; i++)
    ws[i] = s[i];

  ws[i] = 0;
}
#endif


static int
failed (int result)
{
  if ((result != SQL_SUCCESS) &&
      (result != SQL_SUCCESS_WITH_INFO))
    return 1;
  return 0;
}


static const char *
failure_diag (st_sql_t *sql)
{
  SQLSMALLINT mlen;
  unsigned char stat[10];  // SQL status
  unsigned char msg[200];
  static char buf[MAXBUFSIZE];

  SQLGetDiagRec (SQL_HANDLE_DBC, sql->conn, 1,
                 stat, &sql->err, msg, 100, &mlen);

  sprintf (buf, "%s (%d)\n", msg, (int) sql->err);

  return buf; 
}


st_sql_t *
sql_odbc_open (st_sql_t *sql,
               const char *host, int port,
               const char *db, const char *user,
               const char *password)
{
  (void) host;
  (void) port;  // both configured in odbc.ini

  // allocate environment handle and register version 
  if (failed (SQLAllocHandle (SQL_HANDLE_ENV, SQL_NULL_HANDLE, &sql->env))) 
    {
      fprintf (stderr, "ERROR: SQLAllocHandle()\n");
      return NULL;
    }

  if (failed (SQLSetEnvAttr (sql->env, SQL_ATTR_ODBC_VERSION, (void *) SQL_OV_ODBC3, 0)))
    {
      fprintf (stderr, "ERROR: SQLSetEnvAttr()\n");

      SQLFreeHandle (SQL_HANDLE_ENV, sql->env);

      return NULL;
    }

  // allocate connection handle, set timeout
  if (failed (SQLAllocHandle (SQL_HANDLE_DBC, sql->env, &sql->conn)))
    {
      fprintf (stderr, "ERROR: SQLAllocHandle()\n");

      SQLFreeHandle (SQL_HANDLE_ENV, sql->env);

      return NULL;
    }

  SQLSetConnectAttr (sql->conn, SQL_LOGIN_TIMEOUT, (SQLPOINTER *) 5, 0);

  // connect to the db
  if (failed (SQLConnect (sql->conn,
                          (SQLCHAR *) db,
                          SQL_NTS,
                          (SQLCHAR *) user,
                          SQL_NTS,
                          (SQLCHAR *) password,
                          SQL_NTS)))
    {
      fprintf (stderr, "ERROR: SQLConnect(): %s", failure_diag (sql));

      SQLFreeHandle (SQL_HANDLE_ENV, sql->env);

      return NULL;
    }

#if 0
  if (failed (SQLAllocHandle (SQL_HANDLE_STMT, sql->conn, &sql->hstmt)))
    {
      fprintf (stderr, "ERROR: SQLAllocHandle(): %s", failure_diag (sql));

      SQLDisconnect (sql->conn);
      SQLFreeHandle (SQL_HANDLE_DBC, sql->conn);
      SQLFreeHandle (SQL_HANDLE_ENV, sql->env);

      return NULL;
    }
#endif

  return sql;
}


int
sql_odbc_close (st_sql_t *sql)
{
  SQLFreeHandle (SQL_HANDLE_STMT, sql->hstmt);
  SQLDisconnect (sql->conn);
  SQLFreeHandle (SQL_HANDLE_DBC, sql->conn);
  SQLFreeHandle (SQL_HANDLE_ENV, sql->env);

  return 0;
}


static int
sql_odbc_query (st_sql_t *sql, const char *query)
{
  SQLINTEGER rows;
  SQLSMALLINT cols;

#if 1
  if (failed (SQLAllocHandle (SQL_HANDLE_STMT, sql->conn, &sql->hstmt)))
    {
      fprintf (stderr, "ERROR: SQLAllocHandle(): %s", failure_diag (sql));

      SQLDisconnect (sql->conn);
      SQLFreeHandle (SQL_HANDLE_DBC, sql->conn);
      SQLFreeHandle (SQL_HANDLE_ENV, sql->env);

      return -1;
    }
#endif

//  SQLBindCol (sql->hstmt, 1, SQL_C_CHAR, &sql->buffer, 150, &sql->err);
//  SQLBindCol (sql->hstmt, 2, SQL_C_ULONG, &sql->id, 150, &sql->err);

  if (failed (SQLExecDirect (sql->hstmt, (SQLCHAR *) query, SQL_NTS)))
    {
      fprintf (stderr, "ERROR: SQLExecDirect(): %s", failure_diag (sql));

      sql_close (sql);
      return -1;
    }

  if (failed (SQLNumResultCols (sql->hstmt, &cols)))
    {
      sql_close (sql);
      return -1;
    }

  if (failed (SQLRowCount (sql->hstmt, &rows)))
    {
      sql_close (sql);
      return -1;
    }

#ifdef  DEBUG
  printf ("Number of Columns %d\n", (int) cols);
  printf ("Number of Rows %d\n", (int) rows);
#endif

  SQLBindCol (sql->hstmt, 1, SQL_C_CHAR, &sql->buffer, 150, &sql->err);
  SQLBindCol (sql->hstmt, 2, SQL_C_ULONG, &sql->id, 150, &sql->err);
    
  while (SQLFetch (sql->hstmt) != SQL_NO_DATA)
    printf ("Result: %d %s\n", (int) sql->id, sql->buffer);

  return 0;
}


const char *
sql_odbc_gets (st_sql_t *sql, char *buf, int buf_len)
{
  if (SQLFetch (sql->hstmt) == SQL_NO_DATA)
    return NULL;
  return NULL;  
}
#endif  // USE_ODBC
