/*
rsstool.c - RSStool reading, parsing and writing RSS (and ATOM) feeds

Copyright (c) 2006 NoisyB


This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef  HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "misc/itypes.h"
#include "misc/string.h"
#include "misc/property.h"
#include "misc/getopt2.h"
#include "misc/misc.h"
#include "misc/net.h"
#include "misc/rss.h"
#include "rsstool_defines.h"
#include "rsstool.h"
#include "rsstool_misc.h"
#include "rsstool_write.h"


// hacks
#ifdef  USE_HACKS
#include "hack/xxx.h"
#include "hack/troll.h"
#include "hack/torrent.h"
#include "hack/youtube.h"
#include "hack/google.h"
#endif


st_rsstool_t rsstool;


static void rsstool_exit (void)
{
  if (rsstool.log)
    {
//      rsstool_log (&rsstool, "rsstool_exit()");

      if (rsstool.log != stderr && rsstool.log != stdout)
        {
          fclose (rsstool.log);
          rsstool.log = NULL;
        }
    }
}


int
main (int argc, char **argv)
{
//  char short_options[ARGS_MAX];
  struct option long_only_options[ARGS_MAX];
  int option_index = 0;
  int c = 0;
  char buf[MAXBUFSIZE];
  const char *p = NULL;
  FILE *fh = NULL;
#if 0
  st_property_t props[] = {
    {NULL, NULL, NULL}
  };
#endif
  const st_getopt2_t options[] = {
    {
      NULL,      0, 0, 0, NULL,
      "rsstool " RSSTOOL_VERSION_S " " CURRENT_OS_S " 2006 by NoisyB\n"
      "This may be freely redistributed under the terms of the GNU Public License\n\n"
      "Usage: rsstool [OPTION] FILE(S)... URL(S)...\n", NULL
    },
    {
      NULL,      0, 0, 0,
      NULL,   "Read, parse, merge and write RSS (and ATOM) feeds\n"
              "\nSupports: RSS 0.9x, 1.0, 2.0 and ATOM 0.1, 0.2, 0.3\n"
              "\nThe RSS (or ATOM) feeds will be merged and re-sorted if you enter more than\n"
              "one URL or FILEname\n", NULL 
    },
    {
      NULL,      0, 0, 0,
      NULL,   "Options", NULL
    },
    {
      "u", 1, 0, RSSTOOL_U,
      "AGENT", "use user-AGENT for connecting", NULL
    },
#ifdef  USE_ZLIB
    {
      "gzip", 0, 0, RSSTOOL_GZIP,
      NULL, "use gzip compression for downloading", NULL
    },
#endif
#if 0
    {
      "input-file", 1, 0, RSSTOOL_INPUT_FILE,
      "FILE", "download feeds found in FILE", NULL
    },
    {
      "i", 1, 0, RSSTOOL_INPUT_FILE,
      "FILE", "same as " OPTION_LONG_S "input-file", NULL
    },
#endif
    {
      "log", 1, 0, RSSTOOL_LOG,
      "FILE", "write a log to FILE (including HTTP headers)", NULL
    },
    {
      "version", 0, 0, RSSTOOL_VER,
      NULL,   "output version information and exit", NULL
    },
    {
      "ver",     0, 0, RSSTOOL_VER,
      NULL,   NULL, NULL
    },
    {
      "help",    0, 0, RSSTOOL_HELP,
      NULL,   "display this help and exit", NULL
    },
    {
      "h",       0, 0, RSSTOOL_HELP,
      NULL,   NULL, NULL
    },
#if 0
    {
      "q", 0, 0, RSSTOOL_QUIET,
      NULL,   "be quiet/less verbose", NULL
    },
#endif
    {
      NULL, 0, 0, 0,
      NULL,   "\nStrip & Sort", NULL
    },
    {
      "shtml",        0, 0, RSSTOOL_SHTML,
      NULL,   "strip HTML code from descriptions", NULL
    },
    {
      "s",        0, 0, RSSTOOL_SHTML,
      NULL,   "same as " OPTION_LONG_S "shtml (deprecated)", NULL
    },
    {
      "swhite",        0, 0, RSSTOOL_SWHITE,
      NULL,   "strip whitespace from descriptions", NULL
    },
    {
      "slf",        0, 0, RSSTOOL_SLF,
      NULL,   "strip line feeds/carriage returns from descriptions", NULL
    },
    {
      "sdesc",        0, 0, RSSTOOL_SDESC,
      NULL,   "strip the whole descriptions from feeds", NULL
    },
    {
      "sd",        0, 0, RSSTOOL_SDESC,
      NULL,   "same as " OPTION_LONG_S "sdesc (deprecated)", NULL
    },
    {
      "1",       0, 0, RSSTOOL_SDESC,
      NULL,   "same as " OPTION_LONG_S "sdesc (deprecated)", NULL
    },
    {
      "r",        0, 0, RSSTOOL_REVERSE,
      NULL,   "sort reverse", NULL
    },
    {
      NULL, 0, 0, 0,
      NULL,   "\nOutput", NULL
    },
    {
      "html",       0, 0, RSSTOOL_HTML,
      NULL,   "output as html", NULL
    },
    {
      "txt",        0, 0, RSSTOOL_TXT,
      NULL,   "output as plain text", NULL
    },
    {
      "rss",       2, 0, RSSTOOL_RSS,
      "VERSION",   "output as RSS feed\n"
                   "VERSION=1 will write RSS v1.0\n"
                   "VERSION=2 will write RSS v2.0 (default)", NULL
    },
    {
      "bookmarks",       0, 0, RSSTOOL_BOOKMARKS,
      NULL,   "output as bookmarks.html for use with Mozilla or Firefox", NULL
    },
    {
      "csv",       0, 0, RSSTOOL_CSV,
      NULL,   "output as comma-separated values CSV", NULL
    },
    {
      "template", 1, 0, RSSTOOL_TEMPLATE,
      "FILE|URL",   "parse template file and replace tags with content\n"
                    "Tags: \"<rsstool:url item=NUM>\"   url/link of item NUM\n"
                    "      \"<rsstool:title item=NUM>\" title of item NUM\n"
                    "      \"<rsstool:desc item=NUM>\"  description of item\n"
                    "                                   NUM\n"
                    "      \"<rsstool:date item=NUM>\"  date of item NUM\n"
                    "      \"<rsstool:site item=NUM>\"  site where item NUM\n"
                    "                                   came from\n"
                    "      \"<rsstool:start item=NUM>\" start of item NUM\n"
                    "                                   will be replaced with\n"
                    "                                   \"<!--\" if item NUM is\n"
                    "                                   empty\n"
                    "      \"<rsstool:end item=NUM>\"   end of item NUM\n"
                    "                                   will be replaced with\n"
                    "                                   \"-->\" if item NUM is\n"
                    "                                   empty\n"
                    "      \"<rsstool:updated>\"        rsstool notice (Thanks!)", NULL
    },
    {
      "property", 1, 0, RSSTOOL_PROPERTY,
      "FILENAME",   "output as property file\n"
                    "Layout: NAME_NUM=VALUE\n"
                    "        NAME  is \"title\", \"url\", etc..\n"
                    "        NUM   is an integer counting the items\n"
                    "        VALUE is the content", NULL
    },
    {
      "sql", 0, 0, RSSTOOL_SQL,
      NULL,   "output as ANSI SQL script", NULL
    },
#ifdef  USE_MYSQL
    {
      "mysql", 1, 0, RSSTOOL_MYSQL,
      "URL",   "write direct to MySQL DB using MySQL API\n"
               "URL syntax: user:passwd@host:port/database", NULL
    },
#endif
#ifdef  USE_ODBC
    {
      "odbc", 1, 0, RSSTOOL_ODBC,
      "URL",   "write direct to DB using ODBC\n"
               "URL syntax: user:passwd@host:port/database", NULL
    },
#endif
#ifdef  USE_HACKS
    {
      NULL,      0, 0, 0,
      NULL,   "\nHacks\n"
              "generate RSS feed from HTML code of sites that do not offer RSS feeds\n"
              "The code of these hacks might sometimes not work if site owners keep\n"
              "changing the syntax layout of their HTML too frequently\n"
              "You will have to wait for the next update of this tool in such a case\n", NULL
    },
    {
      "href", 1, 0, RSSTOOL_HREF,
      "FILE|URL", "get all links from a HTML document", NULL
    },
    {
      "google", 2, 0, RSSTOOL_GOOGLE,
      "LANG", "search Google for all available RSS feeds in LANG (default: en)\n"
              "LANG must be a ISO 3166-1-alpha-2 code", NULL
    },
    {
      "youtube",   1, 0, RSSTOOL_YOUTUBE,
      "SEARCH",   "generate RSS feed with download links to the latest\n"
                  "videos on Youtube after SEARCHing for them", NULL
    },
    {
      "xxx", 0, 0, RSSTOOL_XXX,
      NULL, "generate RSS feed from several free XXX sites", NULL
    },
#if 0
    {
      "metacafe",   1, 0, RSSTOOL_METACAFE,
      "SEARCH",   "like " OPTION_LONG_S "youtube but for metacafe movies", NULL
    },
    {
      "torrent",   1, 0, RSSTOOL_TORRENT,
      "SEARCH", "SEARCH many torrent trackers for NAME and generate\n"
                "RSS feed with descriptions and links to the seeds", NULL
    },
    {
      "troll",   0, 0, RSSTOOL_TROLL,
      NULL,   "generate RSS feed from the latest Slashdot trolls", NULL
    },
#endif
#endif  // USE_HACKS
    {
      NULL,       0, 0, 0,
      NULL, "\nReport problems/comments/ideas/whinge to noisyb@gmx.net\n", NULL
    },
    {NULL,      0, 0, 0,   NULL,   NULL, NULL}
  };

#if 0
  realpath2 (PROPERTY_HOME_RC ("quh"), quh.configfile);

  result = property_check (quh.configfile, QUH_CONFIG_VERSION, 1);
  if (result == 1) // update needed
    result = set_property_array (quh.configfile, props);
  if (result == -1) // property_check() or update failed
    return -1;
#endif

  if (argc < 2)
    {
      getopt2_usage (options);
      exit (-1);
    }

  atexit (rsstool_exit);

  memset (&rsstool, 0, sizeof (st_rsstool_t));

  strncpy (rsstool.user_agent, RSSTOOL_USER_AGENT_S, MAXBUFSIZE)[MAXBUFSIZE - 1] = 0;

//  getopt2_short (short_options, options, ARGS_MAX);
  getopt2_long_only (long_only_options, options, ARGS_MAX);

  while ((c = getopt_long_only (argc, argv, "", long_only_options, &option_index)) != -1)
    switch (c)
      {
        case RSSTOOL_QUIET:
          rsstool.quiet = 1;
          break;

        case RSSTOOL_VER:
          printf ("rsstool version: %s\n", RSSTOOL_VERSION_S);
          exit (0);

        case RSSTOOL_HELP:
          getopt2_usage (options);
          exit (0);

        case RSSTOOL_REVERSE:
          rsstool.reverse = 1;
          break;

        case RSSTOOL_LOG:
          p = optarg;
          if (p)
            rsstool.log = fopen (p, "a");
//          rsstool_log (&rsstool, "start");
          break;

#ifdef  USE_ZLIB
        case RSSTOOL_GZIP:
          rsstool.gzip = 1;
          break;
#endif

        case RSSTOOL_SHTML:
          rsstool.strip_html = 1;
          break;

        case RSSTOOL_SDESC:
          rsstool.strip_desc = 1;
          break;

        case RSSTOOL_SWHITE:
          rsstool.strip_whitespace = 1;
          break;

        case RSSTOOL_SLF:
          rsstool.strip_lf = 1;
          break;

        case RSSTOOL_U:
          p = optarg;
          if (p)
            strncpy (rsstool.user_agent, p, MAXBUFSIZE)[MAXBUFSIZE - 1] = 0;
          break;

        case RSSTOOL_HTML:
          rsstool.output = RSSTOOL_OUTPUT_HTML;
          break;

        case RSSTOOL_TXT:
          rsstool.output = RSSTOOL_OUTPUT_TXT;
          break;

        case RSSTOOL_BOOKMARKS:
          rsstool.output = RSSTOOL_OUTPUT_BOOKMARKS;
          break;

        case RSSTOOL_CSV:
          rsstool.output = RSSTOOL_OUTPUT_CSV;
          break;

        case RSSTOOL_RSS:
          rsstool.rss_version = 2;
          p = optarg;
          if (p)
            rsstool.rss_version = strtol (optarg, NULL, 10);
          rsstool.output = RSSTOOL_OUTPUT_RSS;
          break;

        case RSSTOOL_PROPERTY:
          rsstool.output = RSSTOOL_OUTPUT_PROPERTY;
          strncpy (rsstool.property, optarg, FILENAME_MAX)[FILENAME_MAX - 1] = 0;
          break;

        case RSSTOOL_TEMPLATE:
          p = optarg;
          if (p)
            if (access (p, F_OK) != 0)
              p = net_http_get_to_temp (p, rsstool.user_agent, rsstool.gzip);

          if (p)
            {
              rsstool.output = RSSTOOL_OUTPUT_TEMPLATE;
              strncpy (rsstool.template, p, FILENAME_MAX)[FILENAME_MAX - 1] = 0;
            }
          else
            fputs ("ERROR: template not found\n", stderr);
          break;

        case RSSTOOL_SQL:
          rsstool.output = RSSTOOL_OUTPUT_SQL;
          break;

#ifdef  USE_ODBC
        case RSSTOOL_ODBC:
          p = optarg;
          if (p)
            {
              rsstool.output = RSSTOOL_OUTPUT_ODBC;
              strncpy (rsstool.dburl, p, FILENAME_MAX)[FILENAME_MAX - 1] = 0;
            }
          else
            fputs ("ERROR: db not found\n", stderr);
          break;
#endif
#ifdef  USE_MYSQL
        case RSSTOOL_MYSQL:
          p = optarg;
          if (p)
            {
              rsstool.output = RSSTOOL_OUTPUT_MYSQL;
              strncpy (rsstool.dburl, p, FILENAME_MAX)[FILENAME_MAX - 1] = 0;
            }
          else
            fputs ("ERROR: db not found\n", stderr);
          break;
#endif

        // hacks
#ifdef  USE_HACKS
        case RSSTOOL_GOOGLE:
          p = optarg;
          hack_google (&rsstool, p ? p : "en");
          break;

        case RSSTOOL_YOUTUBE:
          if (optarg)
            {
              rsstool.output = RSSTOOL_OUTPUT_RSS;
              rsstool.optarg = optarg;
              youtube_get_rss (&rsstool);
            }
          break;

        case RSSTOOL_HREF:
          p = optarg;
          if (p)
            if (access (p, F_OK) != 0)
              p = net_http_get_to_temp (p, rsstool.user_agent, rsstool.gzip);

          if (p)
            rsstool_get_links (p);
          else
            fputs ("ERROR: HTML document not found\n", stderr);
          break;

        case RSSTOOL_XXX:
          rsstool.output = RSSTOOL_OUTPUT_RSS;
          xxx_get_rss (&rsstool);
          break;

        case RSSTOOL_TORRENT:
          rsstool.output = RSSTOOL_OUTPUT_RSS;
          torrent_get_rss (&rsstool);
          break;

        case RSSTOOL_TROLL:
          rsstool.output = RSSTOOL_OUTPUT_RSS;
          troll_get_rss (&rsstool);
          break;
#endif  // USE_HACKS

        default:
          fputs ("Try 'rsstool --help' for more information.", stdout);
          exit (-1);
      }

  if (!optind)
    {
      getopt2_usage (options);
      exit (-1);
    }

  // get and parse the standard feeds
  while (optind < argc)
    {
      int tmp = 0;
      const char *s = NULL;
      const char *feed_url = NULL;

      p = s = argv[optind++];
      if (access (p, F_OK) != 0)
        {
          feed_url = p;
          p = net_http_get_to_temp (p, rsstool.user_agent, rsstool.gzip);
          tmp = 1;
        }

      if (p)
        {
          if (!rsstool.output) // just print
            {
              if ((fh = fopen (p, "r")))
                {
                  while (fgets (buf, MAXBUFSIZE, fh))
                    fputs (buf, stdout);

                  fclose (fh);
                }
            }
          else
            {
              rsstool_parse_rss (&rsstool, feed_url, p);
            }

          if (tmp)
            remove (p);
        }
      else
        {
          sprintf (buf, "could not open/download %s", s);
          rsstool_log (&rsstool, buf);
        }
    }

  if (!rsstool.output)
    return 0;

  if (!rsstool_get_item_count (&rsstool))
    {
      rsstool_log (&rsstool, "no feeds");
      return -1;
    }

  sprintf (buf, "%d feeds", rsstool_get_item_count (&rsstool));
  rsstool_log (&rsstool, buf);

  // sort rss feed
  rsstool_sort (&rsstool);

  if (rsstool.output)
    switch (rsstool.output)
      {
        case RSSTOOL_OUTPUT_HTML:
          rsstool_write_html (&rsstool);
          break;

        case RSSTOOL_OUTPUT_TXT:
          rsstool_write_txt (&rsstool);
          break;

        case RSSTOOL_OUTPUT_SQL:
          rsstool_write_ansisql (&rsstool);
          break;

        case RSSTOOL_OUTPUT_BOOKMARKS:
          rsstool_write_bookmarks (&rsstool);
          break;

        case RSSTOOL_OUTPUT_CSV:
          rsstool_write_csv (&rsstool);
          break;

        case RSSTOOL_OUTPUT_RSS:
          rsstool_write_rss (&rsstool, rsstool.rss_version);
          break;

#ifdef  USE_ODBC
        case RSSTOOL_OUTPUT_ODBC:
          rsstool_write_odbc (&rsstool);
          break;
#endif

#ifdef  USE_MYSQL
        case RSSTOOL_OUTPUT_MYSQL:
          rsstool_write_mysql (&rsstool);
          break;
#endif

        case RSSTOOL_OUTPUT_TEMPLATE:
          if (*rsstool.template)
            rsstool_write_template (&rsstool, rsstool.template);
          else
            fputs ("ERROR: no input template specified\n", stderr);
          break;

        case RSSTOOL_OUTPUT_PROPERTY:
          if (*rsstool.property)
            rsstool_write_property (&rsstool, rsstool.property);
          else
            fputs ("ERROR: no property filename specified\n", stderr);
          break;
    }
 
  return 0;
}
