/*
rsstool_misc.c - miscellaneous functions for RSStool

Copyright (c) 2006 NoisyB


This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef  HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <dirent.h>
#include <unistd.h>
#include <time.h>
#include "misc/itypes.h"
#include "misc/net.h"
#include "misc/string.h"
#include "misc/file.h"          // realpath2()
#include "misc/property.h"
#include "misc/rss.h"
#include "rsstool_defines.h"
#include "rsstool.h"
#include "rsstool_misc.h"


#ifdef  DEBUG
void
rsstool_st_rsstool_t_sanity_check (st_rsstool_t *rt)
{
  int i = 0;

  for (; i < rsstool_get_item_count (rt); i++)
    printf ("pos:%d\n"
            "title: %s\n"
            "url: %s\n"
            "date: %ld\n"
            "desc: %s\n\n",
      i,
      rt->item[i].title,
      rt->item[i].url,
      rt->item[i].date,
      rt->item[i].desc);

  printf ("rsstool_get_item_count(): %d\n\n", rsstool_get_item_count (rt));
}
#endif


static const char *
strip_html_pass (const char * s)
{
  (void) s;
  return "";
}


static st_tag_filter_t strip_html_filter[] = {
  {
    "",
    strip_html_pass
  }
};


char *
rsstool_strip_html (char *html)
{
  net_tag_filter (html, strip_html_filter, 0);
  return html;
}


static char *
rsstool_strip_lf (char *html)
{
  return strrep (html, "\n", " ");
}


static char *
rsstool_strip_whitespace (char *html)
{
  strrep (html, "\t", " ");
  strrep (html, "  ", " ");
  return strrep (html, "  ", " ");
}


int
rsstool_parse_rss (st_rsstool_t *rt, const char *feed_url, const char *file)
{
  st_rss_t *rss = NULL;

  if (!file)
    return -1;

  rss = rss_open (file);

  if (!rss)
    return -1;

  rsstool_add_item (rt, rss, feed_url ? feed_url : file);

  rss_close (rss);

  return 0;
}


int
rsstool_add_item_s (st_rsstool_t *rt,
                    const char *site,
                    const char *feed_url,
                    time_t date,
                    const char *url,
                    const char *title,
                    const char *desc)
{
  int i = 0;
  char site_s[RSSTOOL_MAXBUFSIZE], title_s[RSSTOOL_MAXBUFSIZE],
    desc_s[RSSTOOL_MAXBUFSIZE];

  if (!url)
    return -1;

  if (!(*url))
    return -1;

  strncpy (site_s, site, RSSTOOL_MAXBUFSIZE)[RSSTOOL_MAXBUFSIZE - 1] = 0;
  strncpy (title_s, title, RSSTOOL_MAXBUFSIZE)[RSSTOOL_MAXBUFSIZE - 1] = 0;
  strncpy (desc_s, desc, RSSTOOL_MAXBUFSIZE)[RSSTOOL_MAXBUFSIZE - 1] = 0;

  if (rsstool.strip_desc)
    *desc_s = 0;

  if (rsstool.strip_html)
    {
      rsstool_strip_html (site_s);
      rsstool_strip_html (title_s);
      rsstool_strip_html (desc_s);
    }

  if (rsstool.strip_lf)
    {
      rsstool_strip_lf (site_s);
      rsstool_strip_lf (title_s);
      rsstool_strip_lf (desc_s);
    }

  if (rsstool.strip_whitespace)
    {
      rsstool_strip_whitespace (site_s);
      rsstool_strip_whitespace (title_s);
      rsstool_strip_whitespace (desc_s);
    }

  for (; i < rt->item_count; i++)
    if (!strcmp (rt->item[i].url, url) ||
        !strcmp (rt->item[i].title, title_s))
//        !strcmp (rt->item[i].desc, desc_s)) // there are feeds w/o any descriptions
      return 0; // dupe

  i = rt->item_count;
  strncpy (rt->item[i].site, site_s, RSSTOOL_MAXBUFSIZE)[RSSTOOL_MAXBUFSIZE - 1] = 0;
  strncpy (rt->item[i].feed_url, feed_url, RSSTOOL_MAXBUFSIZE)[RSSTOOL_MAXBUFSIZE - 1] = 0;
  rt->item[i].date = date;
  strncpy (rt->item[i].url, url, RSSTOOL_MAXBUFSIZE)[RSSTOOL_MAXBUFSIZE - 1] = 0;
  strncpy (rt->item[i].title, title_s, RSSTOOL_MAXBUFSIZE)[RSSTOOL_MAXBUFSIZE - 1] = 0;
  strncpy (rt->item[i].desc, desc_s, RSSTOOL_MAXBUFSIZE)[RSSTOOL_MAXBUFSIZE - 1] = 0;

  rt->item_count++;

  return 0;
}


int
rsstool_add_item (st_rsstool_t *rt, st_rss_t *rss, const char *feed_url)
{
  int i = 0;

  for (; i < rss->item_count; i++)
    rsstool_add_item_s (rt, rss->title,
                            feed_url,
                            rss->item[i].date,
                            rss->item[i].url,
                            rss->item[i].title,
                            rss->item[i].desc);

#ifdef  DEBUG
  rsstool_st_rsstool_t_sanity_check (rt);
#endif

  return 0;
}


int
rsstool_get_item_count (st_rsstool_t *rt)
{
  return rt->item_count;
}


static int
rsstool_sort_compare (const void *a, const void *b)
{
  st_rsstool_item_t *x = (st_rsstool_item_t *) a, *y = (st_rsstool_item_t *) b;

  if (rsstool.reverse)
    {
      if (x->date < y->date)
        return 1;
      else
        return 0;
    }

  if (x->date > y->date)
    return 1;
  else
    return 0;
}


int
rsstool_sort (st_rsstool_t * rt)
{
  st_rsstool_item_t *rss = rt->item;

  qsort (rss, rsstool.item_count, sizeof (st_rsstool_item_t), rsstool_sort_compare);

  return 0;
}


const char *
a_pass (const char *s)
{
  fputs (net_tag_get_value (s, "href"), stdout);

  return "";
}


int
rsstool_get_links (const char *file)
{
  FILE *fh = NULL;
  char buf[MAXBUFSIZE];
  int cf = 0;
  st_tag_filter_t f[] = {
    {
      "a",
      a_pass
    },
    {NULL, NULL}
  };

  if ((fh = fopen (file, "rb")))
    {
      while (fgets (buf, MAXBUFSIZE, fh))
        cf = net_tag_filter (buf, f, cf);

      fclose (fh);
    }

  return 0;
}


int
rsstool_log (st_rsstool_t * rt, const char *s)
{
  char buf[32];
  time_t t = time (0);
  FILE *o = rt->log ? rt->log : stderr;

  strftime (buf, 32, "%b %d %H:%M:%S", localtime (&t));

  fprintf (o, "%s rsstool(%d): ", buf, getpid());
  fputs (s, o);
  fputc ('\n', o);
  fflush (o);

  return 0;
}
